/* Zadatak 1A:

  Gusari su stigli s dva broda do otoka na kojem su sakrili blago.
  Svaka skrinja s blagom je numerirana brojem od 1 do n, n <= 20.
  Gusari znaju da brodovi imaju nosivost k_1 i k_2 tona,
  a i-ta krinja ima masu m_i tona.
  Medjutim, drugi brod ima pregrade u prostoru za teret,
  tako da u njega mozemo utovariti maksimalno n/4 skrinja.

    Napisite rekurzivnu funkciju koja ce ispisati sve moguce nacine za
    utovarivanje skrinja u ta dva broda, s time da sve skrinje moraju
    biti utovarene. Dozvoljeno je da jedan od brodova bude prazan.
    Bitno je samo koja skrinja ide u koji brod, a poredak utovara u
    pojedini brod nije bitan.

    Svaki nacin utovara treba ispisati u svom redu, kao uredjeni par od
    dva niza. Prvi niz su indeksi krinja koje idu u prvi brod i analogno
    za drugi niz. Ako je neki niz prazan, ne treba ispisati nista na tom
    mjestu. Ako nema nacina za utovar, ne treba ispisati nista.

  Primjer: Za nosivosti k_1 = 5, k_2 = 3, i 4 skrinje s masama
  m_1 = 1.0, m_2 = 1.5, m_3 = 2.0, m_4 = 2.5,
  svi moguci nacini utovara su: (1 2 3, 4), (1 2 4, 3).

  Napomene: Smijete koristiti jedno polje realnih brojeva koje sadrzi
  mase skrinja i jedno polje cijelih brojeva za spremanje informacija
  o skrinjama kod utovara. Ne smijete koristiti globalne varijable.
  Smijete koristiti pomocne funkcije.
*/

#include <stdio.h>
#include <stdlib.h>

    /* Funkcija za ispis jednog nacina za utovar.
       Sve informacije su kodirane u polju indeks.
       Stvarni indeks objekta (iz teksta zadatka)
       je za 1 veci od indeksa i (za polja).
       Zato pisemo i + 1. */

void ispis_nacina(int n, int index[])
{
    int i;

        /* Zagrada za pocetak para. */
    printf(" (");

        /* Prolaz po polju indeks za prvi kamion.
           Ako je niz prazan, nema ispisa. */
    for (i = 0; i < n; ++i)
        if (index[i] == 1)
            printf(" %d", i + 1);

        /* Zarez izmedju nizova. */
    printf(",");

        /* Prolaz po polju indeks za drugi kamion.
           Ako je niz prazan, nema ispisa. */
    for (i = 0; i < n; ++i)
        if (index[i] == 2)
            printf(" %d", i + 1);

        /* Zagrada za kraj para i prijelaz u novi red. */
    printf(")\n");
}

    /* Rekurzivna funkcija za utovar sljedeceg objekta.
       Indeksi objekata idu od 0 do n - 1 (za polja),
       a indeksi iz zadatka su za jedan veci.
       Argument i je indeks objekta za polja! */

void stavi(int i, int n_2, double ost_1, double ost_2,
           int n, double m[], int index[])
{
    double novi_ost;

        /* Provjera kraja rekurzije <=> svi objekti su vec
           uspjesno stavljeni (uz zadana ogranicenja),
           tj. i = n, pa provjeravamo i >= n. */
    if (i >= n)
            /* Ispisi nacin utovara - poziv funkcije ispis. */
        ispis_nacina(n, index);

    else {    /* Ovdje je i <= n => nastavak rekurzije. */

            /* Pokusaj utovara i-tog objekta u prvi brod.
               Ako stane, utovari ga. */
        novi_ost = ost_1 - m[i];
        if (novi_ost >= 0.0) {
            index[i] = 1;
            stavi(i + 1, n_2, novi_ost, ost_2, n, m, index);
        }

            /* Pokusaj utovara i-tog objekta u drugi brod.
               Ako zadovoljava ogranicenja i stane, utovari ga. */
        if (n_2 + 1 > n / 4) return;    /* Krsi ogranicenje! */
        novi_ost = ost_2 - m[i];
        if (novi_ost >= 0.0) {
            index[i] = 2;
            stavi(i + 1, n_2 + 1, ost_1, novi_ost, n, m, index);
        }
    }

    return;
}

    /* Nerekurzivna gornja funkcija za cijeli posao.
       Priprema argumente za rekurzivnu funkciju i zove ju.
       Ako nema nacina za utovar, nema nikakvog ispisa. */

void utovar(int n, double m[], double k_1, double k_2, int index[])
{
        /* Pocetni argumenti za rekurzivnu funkciju stavi. */
    int i = 0, n_2 = 0;
    double ost_1 = k_1, ost_2 = k_2;

        /* Poziv rekurzivne funkcije stavi. */
    stavi(i, n_2, ost_1, ost_2, n, m, index);

    return;
}

    /* Glavni program za demo. */

int main(void)
{
        /* Zadane nosivosti. */
    double k_1 = 5.0, k_2 = 3.0;
        /* Polje za mase. */
    double m[20] = { 1.0, 1.5, 2.0, 2.5 };
        /* Broj masa u polju. */
    int n = 4;
        /* Pomocno polje za indekse brodova kod utovara. */
    int index[20];

    int i;

        /* Ispis polaznih podataka. */
    printf(" k_1 = %6.1f\n", k_1);
    printf(" k_2 = %6.1f\n", k_2);
    printf(" n = %d\n", n);
    for (i = 0; i < n; ++i)
        printf(" masa[%d] = %6.1f\n", i + 1, m[i]);
    printf("\n");

        /* Poziv gornje funkcije za utovar. */
    utovar(n, m, k_1, k_2, index);

    return 0;
}
