#include <stdio.h>

    /* Pomocne funkcije za demo - inicijalizacija i crtanje slika. */

void inicijaliziraj(char polje[7][13]) {
      /* Samo inicijaliziramo za ispise. */
  int i, j;
  for(i = 0; i < 7; ++i) {
    for(j = 0; j < 13; ++j) {
      polje[i][j] = '=';
    }
  }
}

void printaj(char polje[7][13]) {
  int i, j;
  printf("\n");
  for (i = 0; i < 7; ++i) {
    for (j = 0; j < 13; ++j) {
      printf("%c", polje[i][j]);
    }
    printf("\n");
  }
}

    /* Funkcija za stavljanje novog pravokutnika sa zadanim znakom.
       Koristi se u inicijalizaciji i u (c) dijelu rjesenja. */

void stavi(char polje[7][13], int pi, int pj, int h, int w, char znak) {
      /* Pretpostavljamo da su koordinate (koje stizu kao argumenti) dobre,
         tj. sva 4 vrha pravokutnika se nalaze unutar zadanog polja! */
  int qi = pi-h+1, qj = pj-w+1;  /* Gornji lijevi vrh. */
  int i, j;
  for(i = pi; i >= qi; --i) {
    for(j = pj; j >= qj; --j) {
      polje[i][j] = znak;
    }
  }
}

    /* ********** Rjesenje ********** */

int isprobaj(char polje[7][13], int pi, int pj, int h, int w) {
      /* Provjerava moze li se na tu lokaciju staviti
         pravokutnik visine h i sirine w. */
  int qi = pi-h+1, qj = pj-w+1;  /* Gornji lijevi vrh. */
  int i, j;
      /* Provjera je li gornji lijevi vrh unutar matrice. */
  if (qi < 0 || qj < 0) return 0;
      /* Provjera pravokutnika. */
  for(i = pi; i >= qi; --i) {
    for(j = pj; j >= qj; --j) {
      if (polje[i][j] != '=')
        return 0;
    }
  }
  return 1;
}

void najvece(char polje[7][13], int pi, int pj, int *h, int *w) {
      /* Za dane koordinate pi i pj, trazi pravokutnik
         najvece povrsine kojeg mozemo smjestiti na tu lokaciju.
         Vraca visinu/sirinu tog pravokutnika (ako stane).
         Vraca -1 ako nikakav pravokutnik ne stane na lokaciju. */
  int mh = -1, mw = -1, povrsina = -1;
  int hh, ww;

      /* Petlje za visinu hh i sirinu ww. */
  for (hh = 1; pi-hh+1 >= 0; ++hh)
    for (ww = 1; pj-ww+1 >= 0; ++ww) {
      if (isprobaj(polje, pi, pj, hh, ww) && hh*ww > povrsina) {
        povrsina = hh*ww;
        mh = hh;
        mw = ww;
      }
  }

      /* Vrati visinu i sirinu. */
  *h = mh; *w = mw;
  return;
}

void kupi(char polje[7][13], char znak) {
      /* Pronalazi najveci prostor koji se moze popuniti
         i popunjava ga znakom znak. */

      /* Za svaku lokaciju pi i pj, nalazimo dimenzije
         najveceg pravokutnika kojeg mozemo tamo smjestiti.
         Varijable koje pocinju s max pamte podatke
         za najveci od svih tih pravokutnika. */
  int maxi, maxj, maxh, maxw, maxpov = -1;
  int pi, pj, h, w;
  for (pi = 0; pi < 7; ++pi) {
    for(pj = 0; pj < 13; ++pj) {
      najvece(polje, pi, pj, &h, &w);
      if (h == -1 || w == -1)
        continue;
      if (h*w > maxpov) {
        maxi = pi; maxj = pj;
        maxh = h;  maxw = w;  maxpov = h*w;
      }
    }
  }

  if (maxpov != -1) {
    stavi(polje, maxi, maxj, maxh, maxw, znak);
  }
}

    /* Glavni program za demo. */

int main(void) {
  char polje[7][13];
  int k, l;

      /* Inicijalizacija za prvu sliku. */
  inicijaliziraj(polje);
  stavi(polje, 1, 3, 2, 4, '$');
  stavi(polje, 5, 4, 4, 2, '%');
  stavi(polje, 5, 12, 3, 6, '*');

      /* Slika 1. */
  printf("Pocetno stanje = slika 1:\n");
  printaj(polje);
  printf("\n");

      /* Pozivi funkcija iz rjesenja zadatka. */
  printf(" isprobaj(polje, 4, 6, 2, 2) = %d\n",
           isprobaj(polje, 4, 6, 2, 2));
  printf(" isprobaj(polje, 4, 6, 2, 3) = %d\n",
           isprobaj(polje, 4, 6, 2, 3));

  najvece(polje, 6, 2, &k, &l);
  printf(" najvece(polje, 6, 2, &k, &l): k = %d, l = %d\n", k, l);

  kupi(polje, '1');

      /* Slika 2. */
  printf("\nStanje nakon kupi(polje, '1') = slika 2:\n");
  printaj(polje);

  return 0;
}
