#include <stdio.h>
#include <stdlib.h>

  /* Silazno sortiranje po umnosku. */

    /* Funkcija za umnozak elemenata jednog
       dvodimenzionalnog niza (matrice).
       Po toj funkcijskoj vrijednosti se sortira. */

double f(double **c, int n) {
    double res = 1.0;
    int i, j;
    for(i = 0; i < n; i++)
        for(j = 0; j < n; j++)
            res *= c[i][j];
    return res;
}

    /* Funkcija za zamjenu dva dvodimenzionalna niza
       (dvije matrice), zamjenom pokazivaca na njih. */

void swap_mat(double ***a, double ***b) {
    double **tmp;
    tmp = *a;
    *a = *b;
    *b = tmp;
    return;
}

    /* Funkcija za zamjenu funkcijskih vrijednosti
       (po kojima se sortira). */

void swap_fun(double *a, double *b) {
    double tmp;
    tmp = *a;
    *a = *b;
    *b = tmp;
    return;
}

    /* Trazena funkcija iz zadatka: sortiranje niza
       dvodimenzionalnih nizova (matrica). */

void sort(double ***niznizova, int m, int n) {

        /* Pomocno polje za funkcijske vrijednosti,
           zato da ih ne moramo stalno racunati. */
    double *fun;
    int i, j;

        /* Alokacija pomocnog polja za funkcijske vrijednosti. */
    fun = malloc(m * sizeof(double));

        /* Racunanje funkcijskih vrijednosti za sve clanove
           "glavnog" niza, tj. za svaku matricu reda n. */
    for (i = 0; i < m; i++)
        fun[i] = f(niznizova[i], n);

        /* Sortiranje niza po funkcijskim vrijednostima.
           Silazno = dovedi najveci na i-to mjesto.
           Stalno radi zamjene (najkraci kod). */
    for (i = 0; i < m-1; i++)
        for (j = i+1; j < m; j++)
            if (fun[i] < fun[j]) {

                    /* Zamjena pokazivaca na matrice. */
                swap_mat(niznizova + i, niznizova + j);

                    /* Zamjena pripadnih funkcijskih
                       vrijednosti. */
                swap_fun(fun + i, fun + j);
            }

        /* Dealokacija pomocnog polja za funkcijske vrijednosti. */
    free(fun);

    return;
}

    /* Glavni program za demo. */

    /* Naredbe za citanje svih potrebnih podataka su u KOMENTARIMA
       (to su: m, n i pojedini elementi za m matrica reda n).
       Umjesto toga, za demo koristimo 3D polje od 4 matrice reda 2. */

int main(void) {

        /* Fiksni niz od 4 matrice reda 2. */
    double nizmat[4][2][2] = {
        { {10, 0}, {0, 0} },
        { {1, 2}, {1, 1} },
        { {1, 1}, {1.5, 1} },
        { {1, -1}, {-1, -1} }
    };

    int m, n, i, j, k;

        /* Pokazivac na jedan dvodimenzionalni niz (matricu). */
    double ***mat;

        /* Citanje m i n. */
/*
    printf("m = ");
    scanf("%d", &m);
    printf("n = ");
    scanf("%d", &n);
*/
        /* Umjesto citanja, imamo fiksni niz od 4 matrice reda 2. */
    m = 4; n = 2;

        /* Alokacija polja od m pokazivaca na
           dvodimenzionalne nizove (matrice reda n). */
    mat = malloc(m * sizeof(double **));

        /* Petlja po svim clanovima glavnog niza. */
    for (i = 0; i < m; i++) {

            /* Alokacija polja od n pokazivaca na
               n redova i-tog dvodim. niza (i-te matrice). */
        mat[i] = malloc(n * sizeof(double *));

            /* Petlja po svim redovima i-tog dvodim. niza
               (i-te matrice). */
        for (j = 0; j < n; j++) {

                /* Alokacija polja od n elemenata (double) za
                   j-ti red i-tog dvodim. niza (i-te matrice). */
            mat[i][j] = malloc(n * sizeof(double));

                /* Citanje n elemenata u j-tom redu i-te matrice. */
            for (k = 0; k < n; k++) {

                    /* Citanje elementa mat[i]j][k]. */
/*
                printf("mats[%d][%d][%d] = ", i, j, k);
                scanf("%lf", &mat[i][j][k]);
*/
                    /* Umjesto citanja, uzimamo odgovarajuci
                       element iz fiksnog niza za primjer. */
                mat[i][j][k] = nizmat[i][j][k];
            }
        }
    }

        /* Sortiranje niza dvodimenzionalnih nizova. */
    sort(mat, m, n);

        /* Ispis sortiranog niza po zadanom kriteriju. */
    printf("\n");
    printf("Sortirani niz nizova:\n");
    for (i = 0; i < m; i++) {
        printf("  Matrica[%d]:\n", i+1);

            /* Ispis elemenata i-te matrice. */
        for (j = 0; j < n; j++) {
            printf("    %d. red:", j+1);
            for (k = 0; k < n; k++)
                printf(" %6.2f", mat[i][j][k]);
            printf("\n");
        }
    }

        /* Petlja za dealokaciju svih clanova (matrica) u nizu. */
    for (i = 0; i < m; i++) {

            /* Dealokacija polja pokazivaca na matrice. */
        for (j = 0; j < n; j++)
            free(mat[i][j]);

            /* Dealokacija polja od n pokazivaca
               na redove i-te matrice. */
        free(mat[i]);
    }

        /* Dealokacija polja pokazivaca na matrice. */
    free(mat);

    return 0;
}
